/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: AndroidObservationsUIController.java,v 3e7ea5dbefcb 2010/05/17 15:10:54 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.android.observation;

import javax.annotation.Nonnull;
import java.util.Date;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import it.tidalwave.util.logging.Logger;
import it.tidalwave.bluebill.observation.Observation;
import it.tidalwave.bluebill.observation.ObservationItem;
import it.tidalwave.bluebill.observation.ObservationManager;
import it.tidalwave.bluebill.observation.ObservationSet;
import it.tidalwave.mobile.ActionEvent;
import it.tidalwave.mobile.ActionListener;
import it.tidalwave.mobile.DateUpdater;
import it.tidalwave.bluebill.mobile.observation.DefaultObservationsUIController;
import it.tidalwave.bluebill.mobile.observation.ObservationCountFormat;
import it.tidalwave.bluebill.mobile.observation.ObservationsUI;
import android.app.Activity;
import android.app.DatePickerDialog;
import android.app.Dialog;
import android.app.TimePickerDialog;
import android.content.Intent;
import android.database.DataSetObserver;
import android.view.MenuItem;
import android.widget.ExpandableListView;
import android.widget.ExpandableListView.ExpandableListContextMenuInfo;
import it.tidalwave.bluebill.mobile.android.R;
import it.tidalwave.bluebill.mobile.android.taxonomy.factsheet.TaxonFactSheetControlFlow;
import it.tidalwave.bluebill.mobile.share.Report;
import it.tidalwave.mobile.android.AndroidUtilities;
import it.tidalwave.mobile.android.ControlFlow;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public class AndroidObservationsUIController extends DefaultObservationsUIController
  {
    private static final String CLASS = AndroidObservationsUIController.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    private final Lookup lookup = Lookup.getDefault();

    private final ObservationListAdapter listAdapter;

    private final ObservationSet observationSet;

    private final ObservationsUI ui;
    
    private Observation contextMenuObservation;

    private ObservationItem contextMenuObservationItem;
    
    private final ObservationCountFormat observationCountFormat = new ObservationCountFormat();

    /*******************************************************************************************************************
     *
     * FIXME: pull up, after using ActionListener and ActionEvent from the patched Swing stuff
     *
     ******************************************************************************************************************/
    private final ActionListener changeDateTimeListener = new ActionListener()
      {
        public void actionPerformed (final @Nonnull ActionEvent event)
          {
            final Date date = event.as(DateUpdater.class).update(contextMenuObservation.getDate());
            contextMenuObservation.replace().date(date).apply();
          }
      };

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public AndroidObservationsUIController (final @Nonnull ObservationsUI ui)
      {
        super(ui);
        this.ui = ui;

        observationSet = lookup.lookup(ObservationManager.class).findOrCreateObservationSetById(null);
        listAdapter = new ObservationListAdapter(observationSet);

        listAdapter.registerDataSetObserver(new DataSetObserver()
          {
            @Override
            public void onChanged()
              {
                ui.setFooterText(observationCountFormat.format(listAdapter.getGroupCount()));
              }
          });
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void fireRefresh()
      {
        listAdapter.notifyDataSetChanged();
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public ObservationListAdapter getListAdapter()
      {
        return listAdapter;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    public void startNewObservationSequence()
      {
        logger.info("startNewObservationSequence()");
        super.startNewObservationSequence();
        ControlFlow.start((Activity)ui, AddObservationControlFlow.class);
      }

    /*******************************************************************************************************************
     *
     *
     *
     ******************************************************************************************************************/
    @Override
    public void browseToFactSheet()
      {
        super.browseToFactSheet();
        logger.info("browseToFactSheet()");
        ControlFlow.start((Activity)ui, TaxonFactSheetControlFlow.class);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    protected void shareObservations (final @Nonnull Report report)
      {
        final String pickAService = NbBundle.getMessage(AndroidObservationsUIController.class, "pickAService");
        final String[] recipients = report.getRecipients().toArray(new String[0]);
        final Intent sendIntent = new Intent(Intent.ACTION_SEND);
        sendIntent.putExtra(Intent.EXTRA_EMAIL, recipients);
        sendIntent.putExtra(Intent.EXTRA_PHONE_NUMBER, recipients);
        sendIntent.putExtra(Intent.EXTRA_SUBJECT, report.getSubject());
        sendIntent.putExtra(Intent.EXTRA_TEXT, report.getBody());
        sendIntent.setType(report.getMimeType());
        ((Activity)ui).startActivity(Intent.createChooser(sendIntent, pickAService));
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public boolean onContextItemSelected (final @Nonnull MenuItem item)
      {
        final ExpandableListContextMenuInfo info = (ExpandableListContextMenuInfo)item.getMenuInfo();
        final int type = ExpandableListView.getPackedPositionType(info.packedPosition);
        final int groupPosition = ExpandableListView.getPackedPositionGroup(info.packedPosition);

        switch (type)
          {
            case ExpandableListView.PACKED_POSITION_TYPE_CHILD:
                final int childPosition = ExpandableListView.getPackedPositionChild(info.packedPosition);
                contextMenuObservationItem = listAdapter.getChild(groupPosition, childPosition);

                switch (item.getItemId())
                  {
                    case R.id.deleteObservationItem:
                      deleteObservationItem(contextMenuObservationItem);
                      break;
                  }

                return true;

            case ExpandableListView.PACKED_POSITION_TYPE_GROUP:
                contextMenuObservation = listAdapter.getGroup(groupPosition);

                switch (item.getItemId())
                  {
                    case R.id.editDate:
                    case R.id.editTime:
                      ui.getCommonUITasks().showDialog(item.getItemId());
                      break;
                  }

                return true;
          }

        return false;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    protected Dialog onCreateDialog (final int id)
      {
        switch (id)
          {
            case R.id.editTime:
                return (Dialog)ui.getCommonUITasks().createTimePickerDialog(changeDateTimeListener);

            case R.id.editDate:
                return (Dialog)ui.getCommonUITasks().createDatePickerDialog(changeDateTimeListener);
          }

        throw new RuntimeException("Unknown dialog id: " + id);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    protected void onPrepareDialog (final int id, final @Nonnull Dialog dialog)
      {
        switch (id)
          {
            case R.id.editTime:
                AndroidUtilities.setDate((TimePickerDialog)dialog, contextMenuObservation.getDate());
                break;

            case R.id.editDate:
                AndroidUtilities.setDate((DatePickerDialog)dialog, contextMenuObservation.getDate());
                break;
          }
      }
  }
