/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: TaxonIntentHelper.java,v 3e7ea5dbefcb 2010/05/17 15:10:54 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.android.taxonomy;

import javax.annotation.Nonnull;
import android.content.Context;
import android.content.Intent;
import org.openide.util.Lookup;
import it.tidalwave.util.Id;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.bluebill.taxonomy.Taxon;
import it.tidalwave.bluebill.taxonomy.Taxonomy;
import it.tidalwave.bluebill.mobile.taxonomy.TaxonomyPreferences;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public final class TaxonIntentHelper
  {
    public static final String ACTION_TAXON = "it.tidalwave.bluebill.android.taxonomy.action.";

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private TaxonIntentHelper()
      {
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public static Intent intentFor (final @Nonnull Taxonomy.Type type)
      {
        return new Intent(ACTION_TAXON + type.name());
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public static Intent intentFor (final @Nonnull Taxon taxon)
      throws NotFoundException
      {
        // FIXME: you should really use the Uri, but it fails to find the Intent
        // FIXME: probably it needs a further declaration in the Android manifest
//        return new Intent(ACTION_TAXON, Uri.parse(taxon.getId().stringValue()));
        final Intent intent = new Intent(ACTION_TAXON + taxon.getType().getNarrower().name());
        intent.putExtra("taxonId", taxon.getId().stringValue());
        return intent;
      }

    /*******************************************************************************************************************
     *
     * FIXME: probably useless now?
     *
     ******************************************************************************************************************/
//    @Nonnull
//    public static Intent createIntent (final @Nonnull Taxon taxon,
//                                       final @Nonnull Context context, // FIXME: get it from the Lookup
//                                       final @Nonnull Class<?> activityClass)
//      throws NotFoundException
//      {
//        // FIXME: you should really use the Uri, but it fails to find the Intent
//        // FIXME: probably it needs a further declaration in the Android manifest
////        return new Intent(ACTION_TAXON, Uri.parse(taxon.getId().stringValue()));
//        final Intent intent = new Intent(context, activityClass);
//        intent.putExtra("taxonId", taxon.getId().stringValue());
//        return intent;
//      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public static Taxon getTaxon (final @Nonnull Intent intent)
      throws NotFoundException
      {
        NotFoundException.throwWhenNull(intent, "No Intent");
        final String id = NotFoundException.throwWhenNull(intent.getExtras(), "No extras").getString("taxonId");
        NotFoundException.throwWhenNull(id, "No taxonId");
//        return Lookup.getInstance().getTaxonomy().findTaxonById(new Id(intent.getData().toString()));
        return Lookup.getDefault().lookup(TaxonomyPreferences.class).getTaxonomy().findTaxonById(new Id(id));
      }
  }
